
/// "albgebr.c++"

/// "WARNING"
// **************************************************************************
// **************************************************************************
/*
                      SINGLE MODE OPTICAL FIBER COUPLING

     This program was thinked and realised by Xavier Outhier at the 
   Czech Thechnical University of Prague (Czech Republic) at the Faculty
   of Nuclear Physics in the department of Physical Electronics.

      This program was created in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

     Please contact me when you use it for your own and give me your
   impression about it (facility of use, bugs,...). Here is my e-mail
                      d2set@multimania.com
	  (precise in the subject or body of message to Xavier Outhier)
	      You can also visit the web site of D2SET association
		          http://www.multimania.com/d2set/


                                                Prague, 29th May 1997

                                                Xavier Outhier

*/
// **************************************************************************
// **************************************************************************
//f

/*************************************************************************/
//ce
//ce  NAME OF FILE            : albebr.c++
//ce  VERSION                 : 1.0
//ce  DATE OF CREATION        : 10/07/1997 (Xavier)
//ce  LAST MODIFICATION ON    : 
//ce  LAST CORRECTION ON      : 
//ce  COMPATIBILITY           : 1.0
//ce  STAGE OF VERIFICATION   : none
//ce  DESCRIPTION OF THE FILE : Contains functions for computation of algebra
//ce    formula.
//ce
/*************************************************************************/

/*************************************************************************/
//cm
//cm  List  of modifications :
//cm
/*************************************************************************/

/*************************************************************************/
//cc
//cc  List  of corrections :
//cc
/*************************************************************************/

/// "includes"
#include <complex.h>
#include "algebra.h"
#include "memories.h"
//f

/// "defines"
// macros for easier using of matrixalloc
#define dmatrix(m,n) ((double**)matrixalloc(m,n,sizeof(double)))
#define cmatrix(m,n) ((complex**)matrixalloc(m,n,sizeof(complex)))
//f

/// "albebr_formula"
/*************************************************************************/
//cf
//cf  Name of the function        : albebr_formula
//cf  Version                     : 0.2
//cf  Date of creation            : 16/06/1997 (Xavier)
//cf  Last modification on        : 10/07/1997 (Xavier)
//cf  Last correction on          : 
//cf  Compatibility               : 1.0
//cf  Verification Stage          : (see below)
//cf  Description of the function : Compute coupling efficiency in a fiber
//cf    through a thins lens and from another fiber.
//cf    Formula(5) from "Coupling efficiency of optics in single-mode fiber
//cf    components", R. E. Wagner, W. J. Tomlison, august 1982, vol. 21,
//cf    No. 15, Applied Optic, p 2677 paragraph G. Results.
//cf
/*************************************************************************/
complex **add_cmatrix(int m, int n, complex **matA, complex **matB)
{

  int x, y;

  complex** matrix = cmatrix(m,n); 
  if (!matrix) return NULL;  

  for (x=0;x<m;x++)
     {
      for (y=0;y<n;y++)  
         {
          matrix[x][y] = matA[x][y]+matB[x][y];
         }
     }

  return(matrix);

}
//f

/*************************************************************************/
//cl
//cl  End of file : algebr.c++
//cl
/*************************************************************************/

//f
