
/// "coupling.c++"

/// "WARNING"
// **************************************************************************
// **************************************************************************
/*
                      SINGLE MODE OPTICAL FIBER COUPLING

     This program was thinked and realised by Xavier Outhier at the 
   Czech Thechnical University of Prague (Czech Republic) at the Faculty
   of Nuclear Physics in the department of Physical Electronics.

      This program was created in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

     Please contact me when you use it for your own and give me your
   impression about it (facility of use, bugs,...). Here is my e-mail
                      d2set@multimania.com
	  (precise in the subject or body of message to Xavier Outhier)
	      You can also visit the web site of D2SET association
		          http://www.multimania.com/d2set/


                                                Prague, 29th May 1997

                                                Xavier Outhier

*/
// **************************************************************************
// **************************************************************************
//f

/*************************************************************************/
//ce
//ce  NAME OF FILE            : coupling.c++
//ce  VERSION                 : 1.0
//ce  DATE OF CREATION        : 02/06/1997 (Xavier)
//ce  LAST MODIFICATION ON    : 04/08/1997 (Xavier)
//ce  LAST CORRECTION ON      : 13/06/1997 (Xavier)
//ce  COMPATIBILITY           : 1.0
//ce  STAGE OF VERIFICATION   : (see below)
//ce    compilation with cc -c -fullwarn -Xcpluscomm ---------------> OK
//ce    linkage avec -lm -------------------------------------------> OK
//ce    execution et passage parametres ----------------------------> OK
//ce  DESCRIPTION OF THE FILE : Main program that manage differents sets of
//ce   launching light coupling into a sngle mode optical fiber.
//ce
/*************************************************************************/

/*************************************************************************/
//cm
//cm  List  of modifications :
//cm
//cm   v0.3, 18/06/1997, Xavier:
//cm    # Add of #include <fstream.h>
//cm
//cm   v0.4, 07/07/1997, Xavier:
//cm    # Miscelenous tests with i/o
//cm
//cm   v1.0, 04/08/1997, Xavier:
//cm    # Final version tested and describe in the manual
//cm
/*************************************************************************/

/*************************************************************************/
//cc
//cc  List  of corrections :
//cc
//cc   v0.2, 13/06/1997, Xavier:
//cc    # void main(void) ---> int main(void)
//cc
/*************************************************************************/

/// "includes"
#include <stdlib.h>
#include <stdio.h>
#include "typedef.h"

// For C++ declaration functions
#include <fstream.h>
#include <iostream.h>
#include <complex.h>
#include "algebra.h"
#include "coupl_eff.h"
#include "exit_pupil.h"
#include "memories.h"
#include "params.h"
#include "read_param.h"
//f

// Intrinsic parameters of the fibre

  double n_core;
  double n_clad;
  double d_core;
  double d_clad;
  int fiber_type;

// Position parameters of the input fiber

  point2D eta_i,eta_p;
  double z_i, z_p;
  point3D fr;

// Exipt pupil plane geometrical parameters
 
  struct bound_1D bound_x;
  struct bound_1D bound_y;
  double h_p_sx, h_p_sy;
  double h_p_rx, h_p_ry;
  point2D X_OS_p;

// Free wave length and refractive index of the surrounding medium
   
  double lambda_0;
  double n_medium;

// Linear fibre misalignment 

  point2D delta_eta_p;

// Angular fibre misalignment 

  double d_theta_x,d_theta_y;
             
// Complex field distribution
  complex **psy_p_S, **psy_p_R;
             
// Complex aberration distribution
  complex **W_s_opt, **W_fibre, **W_tot;

//f

/// "main"
/*************************************************************************/
//cf
//cf  Name of the function        : coupling_formula
//cf  Version                     : 1.2
//cf  Date of creation            : 02/06/1997 (Xavier)
//cf  Last modification on        : 11/07/1997 (Xavier)
//cf  Last correction on          : 
//cf  Compatibility               : 1.0
//cf  Verification Stage          : (see below)
//ce    compilation with cc -c -fullwarn -Xcpluscomm ---------------> OK
//ce    linkage avec -lm -------------------------------------------> OK
//ce    execution et passage parametres ----------------------------> OK
//cf  Description of the function : see DESCRIPTION OF THE FILE
//cf
/*************************************************************************/
int main(void)
{

  int x,y;
  int m,n;

  double V,w0;

  double T, C, k, zR;

  double W_1x,W_1y,W_20;
  double x0,y0;

  point2D centreR;
  point2D centreS;

  point2D X_OR_p;

  double Ri2, R_i, R_p;
  double delta_x, delta_y;
  double delta_R_p;

  double normfR;
  double sind_theta_x,sind_theta_y;

  double xe,ye;
  double incrementx,incrementy;

  int warning=0;
  double power;

// Warning

cout<<"******************************************************************"<<endl
    <<"This program compute the coupling efficiency of optics into"<<endl
    <<"a single-mode fibre"<<endl<<endl
    <<"Input data are in the files of the directory DATA_IN, report to"<<endl
    <<"the manual for more precisions"<<endl<<endl;
    
 
// Reading input parameters in files.
   read_caract_light();
   read_intrinsic_fibre();
   read_position_fibre();
   read_source_image();
   read_exit_pupil();

// opening file stream for writing computed paremters
   ofstream rpout("../DATA_OUT/read_params.out");
   if (!rpout) cerr<<"unable to open 'read_params.out' for output";

rpout<<"Light and surrounding medium caracteristics"<<endl
     <<"lambda_0 = "<<lambda_0<<endl
     <<"n_medium = "<<n_medium<<endl<<endl<<flush;

rpout<<"Intrinsic parameters of the fibre"<<endl
     <<"n_core = "<<n_core<<endl
     <<"n_clad = "<<n_clad<<endl
     <<"d_core = "<<d_core<<endl
     <<"d_clad = "<<d_clad<<endl
     <<"fiber_type = "<<fiber_type<<endl<<endl<<flush;

rpout <<"Relative positions parameters between the receiving fibre and the"
      <<" exit-pupil plane"<<endl
      <<"eta_i[0]  = "<<eta_i[0]<<endl
      <<"eta_i[1]  = "<<eta_i[1]<<endl
      <<"z_i  = "<<z_i<<endl
      <<"fr[0]  = "<<fr[0]<<endl
      <<"fr[1]  = "<<fr[1]<<endl
      <<"fr[2]  = "<<fr[2]<<endl<<endl<<flush;

rpout <<"Position of the source image point"<<endl
      <<"eta_p[0] = "<<eta_p[0]<<endl
      <<"eta_p[1] = "<<eta_p[1]<<endl
      <<"z_p = "<<z_p<<endl<<endl<<flush;

rpout <<"Exit pupil window parameters and size of the incoming beam"<<endl
      <<"bound_x.num  = "<<bound_x.num<<endl
      <<"bound_y.num  = "<<bound_y.num<<endl
      <<"bound_x.min = "<<bound_x.min<<endl
      <<"bound_x.max = "<<bound_x.max<<endl
      <<"bound_y.min = "<<bound_y.min<<endl
      <<"bound_y.max = "<<bound_y.max<<endl
      <<"h_p_sx = "<<h_p_sx<<endl
      <<"h_p_sy = "<<h_p_sy<<endl
      <<"X_OS_p[0] = "<<X_OS_p[0]<<endl
      <<"X_OS_p[1] = "<<X_OS_p[1]<<endl<<endl<<flush;

// Check matching of the window and of the size of the beam
   if ( ((X_OS_p[0]-2.*h_p_sx) < bound_x.min) ||
        ((X_OS_p[0]+2.*h_p_sx) > bound_x.max) ||
        ((X_OS_p[1]-2.*h_p_sy) < bound_y.min) ||
        ((X_OS_p[1]+2.*h_p_sy) > bound_y.max) ) 
     {
      warning = 1;
      cerr<<"*************** WARNING MESSAGE *******************"<<endl;
      cerr<<" Result may be not very numerically precise due to"<<endl
          <<" the size of the INCOMING BEAM compared with the"<<endl
          <<" window size."<<endl
          <<" Please check the differents values of Exit pupil"<<endl
          <<" window parameters in the file read_params.out."<<endl;
      cerr<<"***************************************************"<<endl<<endl;
     }

// Computation of others parameters 

// Normalization of fR

   normfR = sqrt( (fr[0]*fr[0]) + (fr[1]*fr[1]) + (fr[2]*fr[2]) );
   fr[0] /= normfR;
   fr[1] /= normfR;
   fr[2] /= normfR;

   delta_R_p = sqrt(eta_i[0]*eta_i[0] + eta_i[1]*eta_i[1] + z_i*z_i)
              -sqrt(eta_p[0]*eta_p[0] + eta_p[1]*eta_p[1] + z_p*z_p);

   sind_theta_x = fr[0];
   sind_theta_y = fr[1];

   V = V_norm(n_core, n_clad , lambda_0, d_core/2.);

   if (fiber_type == 0)
      {
       w0 = wf_Vstep_a(V, d_core/2.);
      }
   else
      {
       w0 = wf_Vparab_a(V, d_core/2.);
      }

   zR = z_R(n_medium,lambda_0,w0);

   Ri2 = eta_i[0]*eta_i[0]
        +eta_i[1]*eta_i[1]
        +z_i*z_i;

   R_i = R_p = sqrt(Ri2);

// Formula (A12) p2686!
   delta_x = (eta_i[0]-eta_p[0]) - delta_R_p*(eta_p[0]/R_p);
   delta_y = (eta_i[1]-eta_p[1]) - delta_R_p*(eta_p[1]/R_p);

   h_p_rx = h_p_ry = w_gauss(Ri2, w0,  zR);

// Under formula (A19) p 2687
   X_OR_p[0] = eta_i[0] + fr[0]*R_i;
   X_OR_p[1] = eta_i[1] + fr[1]*R_i;

// Check matching of the window and of the size of the beam

   if ( ((X_OR_p[0]-2.*h_p_rx) < bound_x.min) ||
        ((X_OR_p[0]+2.*h_p_rx) > bound_x.max) ||
        ((X_OR_p[1]-2.*h_p_ry) < bound_y.min) ||
        ((X_OR_p[1]+2.*h_p_ry) > bound_y.max) ) 
     {
      warning = 2;
      cerr<<"*************** WARNING MESSAGE *******************"<<endl;
      cerr<<" Result may be not very numerically precise due to"<<endl
          <<" the size of the RECEIVING FIBRE BEAM compared with the"<<endl
          <<" window size."<<endl
          <<" Please check the differents values of exit pupil"<<endl
          <<" window parameters in the files read_params.out and"<<endl
          <<" computed_params.out."<<endl;
      cerr<<"***************************************************"<<endl<<endl;
     }

   centreR[0] = X_OR_p[0];
   centreR[1] = X_OR_p[1];
   centreS[0] = X_OS_p[0];
   centreS[1] = X_OS_p[1];

   k = 2.*M_PI/lambda_0;

// opening file stream for writing computed paremters
   ofstream cpout("../DATA_OUT/computed_params.out");
   if (!cpout) cerr<<"Unable to open 'computed_params.out' for output";

cpout<<"Computed parameters"<<endl<<endl
     <<"V = "<<V<<endl
     <<"w0 = "<<w0<<endl
     <<"zR = "<<zR<<endl
     <<"Ri2 = "<<Ri2<<endl
     <<"R_i = "<<R_i<<endl
     <<"R_p = "<<R_p<<endl
     <<"h_p_rx = "<<h_p_rx<<endl
     <<"h_p_ry = "<<h_p_ry<<endl
     <<"X_OR_p[0] = "<<X_OR_p[0]<<endl
     <<"X_OR_p[1] = "<<X_OR_p[1]<<endl
     <<"centreS[0] = "<<centreS[0]<<endl
     <<"centreS[1] = "<<centreS[1]<<endl
     <<"fr normalized: "<<endl
     <<"fr[0]  = "<<fr[0]<<endl
     <<"fr[1]  = "<<fr[1]<<endl
     <<"fr[2]  = "<<fr[2]<<endl
     <<"delta_R_p = "<<delta_R_p<<endl
     <<"sind_theta_x = "<<sind_theta_x<<endl
     <<"sind_theta_y = "<<sind_theta_y<<endl
     <<"delta_x = "<<delta_x<<endl
     <<"delta_y = "<<delta_y<<endl
     <<"delta_R_p = "<<delta_R_p<<endl
     <<"centreR[0] = "<<centreR[0]<<endl
     <<"centreR[1] = "<<centreR[1]<<endl
     <<"k = "<<k<<endl<<flush;

// Compute fibre misalignment coefficients W_1x, W_1y and W_20
//  from input data
   coef_lin_fib_misalign(delta_x,delta_y,
                         R_p,delta_R_p,
                         &W_1x,&W_1y,&W_20);

   cpout << "W_1x =" <<W_1x<<endl
        << "W_1y =" <<W_1y<<endl
        << "W_20 =" <<W_20<<endl<<flush;

// read/compute h's_x, h's_y, psy_p_S

// Memory allocation
  m = bound_x.num;
  n = bound_y.num;
  field_aberr_alloc(m,n);

// Compute psy_p_S
  psy_p_S = psy_p_S_formula(h_p_sx,h_p_sy,centreS,bound_x,bound_y);

// Compute and print if warning!=0 the total power that must be 1.000
  power = abs(power_field(psy_p_S,bound_x,bound_y));
  cpout<<"warning = "<<warning<<endl
       <<"The total power of psy_p_S is "<<power<<endl<<flush;
  if (warning==1) {cerr<<" The total power of psy_p_S is "<<power<<endl;}

// compute unnormalized exit-pupil shift

   angl_fib_misalign(sind_theta_x, sind_theta_y, R_p, &x0, &y0);

// unnormalize them to put in centre_R

   centreR[0] += x0;
   centreR[1] += y0;

   cpout<<"After have taken into account of the angular fibre misalignment"<<endl
        <<"centreR[0] =" <<centreR[0]<<endl
        <<"centreR[1] =" <<centreR[1]<<endl;

// Compute psy_p_R
  psy_p_R = psy_p_R_formula(h_p_rx,h_p_ry,centreR,bound_x,bound_y);

// Compute and print the total power that must be 1.000
  power = abs(power_field(psy_p_R,bound_x,bound_y));
  cpout<<"The total power of psy_p_R is "<<power<<flush<<endl;
  if (warning==2) {cerr<<" The total power of psy_p_R is "<<power<<endl;}

// Read total aberration of the optic system: W_s_opt


// W_s_opt = W_s_opt_read();

// Compute aberration dur to misalignment of the receiving fibre: W_fibre
   W_fibre = misalign_aberration(W_1x,W_1y,W_20,
                                 bound_x,bound_y,x0,y0);
   
// Add W_s_opt and  W_fibre in W_tot
   W_tot = add_cmatrix(m,n,W_s_opt,W_fibre);

   ofstream f_ep("../DATA_OUT/field_exit_pup.out");
   if (!f_ep) cerr<<"Unable to open '../DATA_OUT/field_exit_pup.out' for output";
  incrementx = (bound_x.max-bound_x.min)/m;
  incrementy = (bound_y.max-bound_y.min)/m;

  xe = bound_x.min - incrementx/2.;

  for (x=0;x<m;x++)
     {
     xe += incrementx;
     ye = bound_y.min - incrementy/2.;
      for (y=0;y<n;y++)
         {
         ye += incrementy;
         f_ep<< xe <<" "<< ye <<" "
             << real(W_tot[x][y]) <<" "
             << imag(W_tot[x][y]) <<" "
             << real(psy_p_S[x][y]) <<" "
             << imag(psy_p_S[x][y]) <<" "
             << real(psy_p_R[x][y]) <<" " 
             << imag(psy_p_R[x][y]) <<endl;
         }
      f_ep<< "\n"<<flush;
     }

// Actually compute the coupling efficiency with index matching

  C = coupl_eff_formula(psy_p_S,k,W_tot,psy_p_R,bound_x,bound_y);

// Transmission factor at the fiber endface for index mismatch

  T = reflect_power_coef(n_medium, (n_clad+n_core)/2.);
  C =T*C;

// Output data

   ofstream effile("../DATA_OUT/val_coupl_eff.out");
   if (!effile) cerr<<"Pb with opening ../DATA_OUT/val_coupl_eff.out file";

// in percent
  effile <<100.*C<<" "<<flush;
// in dB
  C =10.*log10(C);
  effile <<C<<"\n"<<flush;


// Free memory
  field_aberr_free(m,n);

// Wanring of end od the code

cout<<endl
    <<"The computation are finished. You can see results in the directory"<<endl
    <<"DATA_OUT. The coupling efficiency in % and in dB is in the file"<<endl
    <<"val_coupl_eff.out"<<endl
    <<"You can check input data in the file read_params.out"<<endl
    <<"Computed parameters areavailable in the file computed_params.out"<<endl
    <<"Finally complex field distributions use for the computation of the"<<endl
    <<"overlap-integral (total aberration, source and mode pattern of "<<endl
    <<" the fibre) are in ASCII format in the file field_exit_pup.out"<<endl
    <<endl
    <<"The coupling efficiency for your configuration is:"<<endl
    <<"                     C = "<<C<<" dB."<<endl<<endl
    <<"******************************************************************"<<endl
    <<endl<<flush;

  return 0;
}
//f

/*************************************************************************/
//cl
//cl  End of file : coupling.c++
//cl
/*************************************************************************/
//f
