
/// "exit_pupil.c++"

/// "WARNING"
// **************************************************************************
// **************************************************************************
/*
                      SINGLE MODE OPTICAL FIBER COUPLING

     This program was thinked and realised by Xavier Outhier at the 
   Czech Thechnical University of Prague (Czech Republic) at the Faculty
   of Nuclear Physics in the department of Physical Electronics.

      This program was created in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

     Please contact me when you use it for your own and give me your
   impression about it (facility of use, bugs,...). Here is my e-mail
                      d2set@multimania.com
	  (precise in the subject or body of message to Xavier Outhier)
	      You can also visit the web site of D2SET association
		          http://www.multimania.com/d2set/


                                                Prague, 29th May 1997

                                                Xavier Outhier

*/
// **************************************************************************
// **************************************************************************
//f

/*************************************************************************/
//ce
//ce  NAME OF FILE            : exit_pupil.c++
//ce  VERSION                 : 2.4
//ce  DATE OF CREATION        : 10/07/1997 (Xavier)
//ce  LAST MODIFICATION ON    : 25/07/1997 (Xavier)
//ce  LAST CORRECTION ON      : 15/07/1997 (Xavier)
//ce  COMPATIBILITY           : 1.0
//ce  STAGE OF VERIFICATION   : none
//ce  DESCRIPTION OF THE FILE : Contains functions for computation or reading
//ce    the far-field distribution of the receiving fiber: exit_pupil_R and of
//ce    the source afetr the optic system: exit_pupil_S.
//ce    Reference: "Coupling efficiency of optics in single-mode fiber
//ce    components", R. E. Wagner and W. J. Tomlinson, August 1982, Vol. 21
//ce    No. 15, Applied Optics.
//ce
//ce  Thisa file is the fusion of exit_pupil_S.c++ and exit_pupil_R.c++.
//ce
/*************************************************************************/

/*************************************************************************/
//cm
//cm  List  of modifications :
//cm
//cm   v2.2, 23/07/1997, Xavier:
//cm    # gaussian_ffield elliptic section!
//cm
//cm   v2.3, 24/07/1997, Xavier:
//cm    # Two parameters h_p_rx and h_p_ry for Gaussian with elliptic 
//cm      section for psy_p_R_formula().
//cm    # Two parameters h_p_sx and h_p_sy for Gaussian with elliptic 
//cm      section for psy_p_S_formula().
//cm
/*************************************************************************/

/*************************************************************************/
//cc
//cc  List  of corrections :
//cc
//cc   v2.1, 15/07/1997, Xavier:
//cc    # xpe and ype become x and y in misalign_aberration formulae.
//cc
/*************************************************************************/

/// "includes"
#include <complex.h>
#include "typedef.h"
#include "exit_pupil.h"
#include "memories.h"
//f

/// "defines"
// macros for easier using of matrixalloc
#define dmatrix(m,n) ((double**)matrixalloc(m,n,sizeof(double)))
#define cmatrix(m,n) ((complex**)matrixalloc(m,n,sizeof(complex)))
//f


/// "External variables"
  extern complex **near_field_S_tmp;
  extern complex **near_field_R_tmp;
  extern complex **far_field_S_tmp;

//f

/// "psy_p_R_formula"
/*************************************************************************/
//cf
//cf  Name of the function        : psy_p_R_formula
//cf  Version                     : 0.4
//cf  Date of creation            : 16/06/1997 (Xavier)
//cf  Last modification on        : 24/07/1997 (Xavier)
//cf  Last correction on          : 
//cf  Compatibility               : 1.0
//cf  Verification Stage          : (see below)
//cf  Description of the function : Compute coupling efficiency in a fiber
//cf    through a thins lens and from another fiber.
//cf
//cf   NOT EFFECTIVE FORMULA
//cf
//cf    Formula from "Coupling efficiency of optics in single-mode fiber
//cf    components", R. E. Wagner, W. J. Tomlison, august 1982, vol. 21,
//cf    No. 15, Applied Optic, p 2677 paragraph G. Results.
//cf
/*************************************************************************/
complex **psy_p_R_formula(double h_p_rx, double h_p_ry, point2D centre,
                         struct bound_1D bound_xpe, struct bound_1D bound_ype )

{
  int n,m;  // Number of colomns and rows

  n = bound_xpe.num;
  m = bound_ype.num;

  complex** matrix = cmatrix(n,m); 
  if (!matrix) return NULL;

//cout<<"\n Dans **exit_pupil_R_formula(...) \n"<<flush;
   matrix = gaussian_ffield(h_p_rx,h_p_ry,centre,bound_xpe,bound_ype);

//cout<<"\n xcentre = "<<x_centre<<" ycentre = "<<y_centre<<"\n"<<flush;


  return(matrix);
}
//f

/// "psy_p_S_formula"
/*************************************************************************/
//cf
//cf  Name of the function        : psy_p_S_formula
//cf  Version                     : 0.4
//cf  Date of creation            : 16/06/1997 (Xavier)
//cf  Last modification on        : 24/07/1997 (Xavier)
//cf  Last correction on          : 
//cf  Compatibility               : 1.0
//cf  Verification Stage          : (see below)
//cf  Description of the function : Compute the far-field of the source
//cf    Reference: "Coupling efficiency of optics in single-mode fiber
//cf    components", R. E. Wagner and W. J. Tomlinson, August 1982, Vol. 21,
//cf    No. 15, Applied Optics.
//cf
/*************************************************************************/
complex **psy_p_S_formula(double h_p_sx, double h_p_sy, point2D centre,
                         struct bound_1D bound_xpe, struct bound_1D bound_ype )

{
  int n,m;  // Number of colomns and rows

  m = bound_xpe.num;
  n = bound_ype.num;

  complex** matrix = cmatrix(m,n); 
  if (!matrix) return NULL;

//cout<<"\n Dans **exit_pupil_S_formula(...) \n"<<flush;
   matrix = gaussian_ffield(h_p_sx, h_p_sy,centre,bound_xpe,bound_ype);

//cout<<"\n xcentre = "<<x_centre<<" ycentre = "<<y_centre<<"\n"<<flush;

  return(matrix);
}
//f

/// "gaussian_ffield"
/*************************************************************************/
//cf
//cf  Name of the function        : gaussian_ffield
//cf  Version                     : 0.3
//cf  Date of creation            : 17/06/1997 (Xavier)
//cf  Last modification on        : 23/07/1997 (Xavier)
//cf  Last correction on          : 04/07/1997 (Xavier)
//cf  Compatibility               : 1.0
//cf  Verification Stage          : (see below)
//cf  Description of the function : Compute a very simple complex-integral 
//cf   of complex parameters in a square domain. Beware of limits!!!
//cf
/*************************************************************************/
complex **gaussian_ffield(double hx, double hy, point2D centre,
                     struct bound_1D bound_xpe, struct bound_1D bound_ype )
{
  int xpe, ype;
  int m,n;
  double one_hx_2, one_hy_2, sqrt1_hxhy;;
  double x, y, dx,dy;

//cout<<"\n Dans **gaussian_ffield(...) "<<flush;
//cout<<"\n xcentre = "<<x_centre<<" ycentre = "<<y_centre<<flush;

  m = bound_xpe.num;
  n = bound_ype.num;
  dx = (bound_xpe.max - bound_xpe.min)/bound_xpe.num;
  dy = (bound_ype.max - bound_ype.min)/bound_ype.num;

  one_hx_2 = 1./(hx*hx);
  one_hy_2 = 1./(hy*hy);

  sqrt1_hxhy = 1./(sqrt(hx)*sqrt(hy));

  complex** matrix = cmatrix(m,n); 
  if (!matrix) return NULL;

  for (xpe=0;xpe<m;xpe++)
     {
      x = bound_xpe.min + (xpe+0.5)*dx;
      x -= centre[0];

      for (ype=0;ype<n;ype++)  
         {

           y = bound_ype.min + (ype+0.5)*dy;
           y -= centre[1];
           matrix[xpe][ype] = M_SQRT1_2*M_2_SQRTPI*sqrt1_hxhy*
                          exp( -( (x*x)*one_hx_2 + (y*y)*one_hy_2 ) );
         }
     }

return(matrix);

}
//f

/// "misalign_aberration"
/*************************************************************************/
//cf
//cf  Name of the function        : misalign_aberration
//cf  Version                     : 0.3
//cf  Date of creation            : 11/07/1997 (Xavier)
//cf  Last modification on        : 31/07/1997 (Xavier)
//cf  Last correction on          : 22/07/1997 (Xavier)
//cf  Compatibility               : 1.0
//cf  Verification Stage          : (see below)
//cf  Description of the function : Compute the fibre misalignment aberration 
//cf   from fibre misalignment coefficients W_1x, W_1y, W_20.
//cf
//cf    Formula(12) from "Coupling efficiency of optics in single-mode fiber
//cf    components", R. E. Wagner, W. J. Tomlison, august 1982, vol. 21,
//cf    No. 15, Applied Optic, p 2679 paragraph G. Results.
//cf
//cf
/*************************************************************************/
complex **misalign_aberration(double W_1x, double W_1y, double W_20,
                     struct bound_1D bound_xpe, struct bound_1D bound_ype,
                     double x0, double y0 )
{
  int xpe, ype;
  int m,n;
  double x, y, dx,dy;

  m = bound_xpe.num;
  n = bound_ype.num;
  dx = (bound_xpe.max - bound_xpe.min)/(bound_xpe.num-1);
  dy = (bound_ype.max - bound_ype.min)/(bound_ype.num-1);

  complex** matrix = cmatrix(m,n);
  if (!matrix)
     {
      cout<<"Error in dynamic memory allocation in function"
          <<" **misalign_aberration()\n";
      return (NULL);
     }

  for (xpe=0;xpe<m;xpe++)
     {
      x = (bound_xpe.min + xpe*dx) - x0;
      for (ype=0;ype<n;ype++)  
         {
           y = (bound_ype.min + ype*dy) -y0;

           matrix[xpe][ype] = W_1x*x + W_1y*y + W_20*(x*x + y*y);
         }
     }


  return(matrix);
}
//f

/*************************************************************************/
//cl
//cl  End of file : exit_pupil.c++
//cl
/*************************************************************************/

//f
