
/// "memories.c++"

/// "WARNING"
// **************************************************************************
// **************************************************************************
/*
                      SINGLE MODE OPTICAL FIBER COUPLING

     This program was thinked and realised by Xavier Outhier at the 
   Czech Thechnical University of Prague (Czech Republic) at the Faculty
   of Nuclear Physics in the department of Physical Electronics.

      This program was created in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

     Please contact me when you use it for your own and give me your
   impression about it (facility of use, bugs,...). Here is my e-mail
                      d2set@multimania.com
	  (precise in the subject or body of message to Xavier Outhier)
	      You can also visit the web site of D2SET association
		          http://www.multimania.com/d2set/


                                                Prague, 29th May 1997

                                                Xavier Outhier

*/
// **************************************************************************
// **************************************************************************
//f

/*************************************************************************/
//ce
//ce  NAME OF FILE            : memories.c++
//ce  VERSION                 : 0.3
//ce  DATE OF CREATION        : 18/06/1997 (Xavier)
//ce  LAST MODIFICATION ON    : 25/07/1997 (Xavier)
//ce  LAST CORRECTION ON      :
//ce  COMPATIBILITY           : 1.0
//ce  STAGE OF VERIFICATION   : none
//ce  DESCRIPTION OF THE FILE : Contains functions that manage dynamic 
//ce    memory allocate and free.
//ce
/*************************************************************************/

/*************************************************************************/
//cm
//cm  List  of modifications :
//cm
//cm   v0.2, 08/07/1997, Xavier:
//cm    # Add of matrixalloc
//cm
//cm   v0.3, 25/07/1997, Xavier:
//cm    # Suppression of useless function
//cm
/*************************************************************************/

/*************************************************************************/
//cc
//cc  List  of corrections :
//cc
/*************************************************************************/

/// "includes"
#include <stdlib.h>
#include <stdio.h>
#include <complex.h>

// For C declaration funcions
#ifdef __cplusplus
extern "C" {
#endif

#ifdef __cplusplus
}
#endif

// For C++ declaration funcions
#include "memories.h"
//f

/// "defines"
// macros for easier using of matrixalloc
#define dmatrix(m,n) ((double**)matrixalloc(m,n,sizeof(double)))
#define cmatrix(m,n) ((complex**)matrixalloc(m,n,sizeof(complex)))

//f

/// "Declaration extern variables"
  extern complex **psy_p_S;
  extern complex **psy_p_R;
  extern complex **W_s_opt;
  extern complex **W_fibre;
  extern complex **W_tot;

//f

/// "matrixalloc"
/*************************************************************************/
//cf
//cf  Name of the function        : matrixalloc
//cf  Version                     : 1.0
//cf  Date of creation            : ? (Jarda)
//cf  Last modification on        : 
//cf  Last correction on          : 
//cf  Compatibility               : 1.0
//cf  Verification Stage          : none
//cf  Description of the function : allocate memory for matrix
//cf
/*************************************************************************/
void** matrixalloc(size_t m,size_t n,size_t elemSize)
{
  char** pp;
  size_t ns;
  size_t i;
  ns=n*elemSize;

  // allocate memory for m*n elems + m* ptr to elems
  pp=(char**)malloc(m*(sizeof(char*)+ns)); 
  if (!pp) return NULL;
  pp[0]=(char*)(pp+m);
  for (i=1; i<m; i++) pp[i]=pp[i-1]+ns;
  return (void**)pp;
}

/// "field_aberr_alloc"
/*************************************************************************/
//cf
//cf  Name of the function        : field_aberr_alloc
//cf  Version                     : 1.0
//cf  Date of creation            : 16/06/1997 (Xavier)
//cf  Last modification on        : 10/07/1997 (Xavier)
//cf  Last correction on          : 
//cf  Compatibility               : 1.0
//cf  Verification Stage          : compilation
//cf  Description of the function : Dynamical allocation for far-field and
//cf    and aberration 2D tabular.
//cf
/*************************************************************************/
void field_aberr_alloc(int n_x, int n_y)
{
  int i;

  // alloc memoire pour psy_p_S psy_p_R 

  psy_p_S = (complex **) malloc ( (n_x)*sizeof(complex *) );
  psy_p_R = (complex **) malloc ( (n_x)*sizeof(complex *) );
  W_s_opt = (complex **) malloc ( (n_x)*sizeof(complex *) );
  W_fibre = (complex **) malloc ( (n_x)*sizeof(complex *) );
  W_tot = (complex **) malloc ( (n_x)*sizeof(complex *) );

  if ((psy_p_S==NULL) || (psy_p_R==NULL)
      || (!W_s_opt) || (!W_fibre) || (!W_tot))
    {
        fprintf(stderr,"Interruption 1: Memory allocation on psy_p_S or");
        fprintf(stderr,"psy_p_Ri or W_s_opt or W_fibre or W_tot \n");
        fprintf(stderr,"in function field_aberr_alloc of file memories.c++ \n");
        exit(1);
    }

  for (i=0;i<n_x;i++)
    {
        psy_p_S[i]= (complex *) malloc ( (n_y)*sizeof(complex) );
        psy_p_R[i]= (complex *) malloc ( (n_y)*sizeof(complex) );
        W_s_opt[i]= (complex *) malloc ( (n_y)*sizeof(complex) );
        W_fibre[i]= (complex *) malloc ( (n_y)*sizeof(complex) );
        W_tot[i]= (complex *) malloc ( (n_y)*sizeof(complex) );

        if ((psy_p_S[i]==NULL) || (!psy_p_R[i])
             || (!W_s_opt[i]) || (!W_fibre[i]) || (!W_tot[i]))
          {
        fprintf(stderr,"Interruption 1: Memory allocation on psy_p_S* or");
        fprintf(stderr,"psy_p_R* or W_s_opt* or W_fibre* or W_tot*\n");
        fprintf(stderr,"in function result_var_alloc of file memories.c++ \n");
        fprintf(stderr,"i= %d (n_x=%d)\n",i,n_x);
        exit(1);
          }
    }


}
//f

/// "field_aberr_free"
/*************************************************************************/
//cf
//cf  Name of the function        : field_aberr_free
//cf  Version                     : 1.0
//cf  Date of creation            : 16/06/1997 (Xavier)
//cf  Last modification on        : 10/07/1997 (Xavier)
//cf  Last correction on          : 
//cf  Compatibility               : 1.0
//cf  Verification Stage          : compilation
//cf  Description of the function : Memoru liberation of differents tabular
//cf    of results_var.
//cf
/*************************************************************************/
void field_aberr_free(int n_x, int n_y)
{
  int i;

  // free memory of psy_p_S, psy_p_R 

  for (i=0;i<n_x;i++)
    {
     free(psy_p_S[i]);
     free(psy_p_R[i]);
     free(W_s_opt[i]);
     free(W_fibre[i]);
     free(W_tot[i]);
    }

  free(psy_p_S);
  free(psy_p_R);
  free(W_s_opt);
  free(W_fibre);
  free(W_tot);

}
//f

/*************************************************************************/
//cl
//cl  End of file : memories.c++
//cl
/*************************************************************************/
//f


