
/// "read_param.c++"

/// "WARNING"
// **************************************************************************
// **************************************************************************
/*
                      SINGLE MODE OPTICAL FIBER COUPLING

     This program was thinked and realised by Xavier Outhier at the 
   Czech Thechnical University of Prague (Czech Republic) at the Faculty
   of Nuclear Physics in the department of Physical Electronics.

      This program was created in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

     Please contact me when you use it for your own and give me your
   impression about it (facility of use, bugs,...). Here is my e-mail
                      d2set@multimania.com
	  (precise in the subject or body of message to Xavier Outhier)
	      You can also visit the web site of D2SET association
		          http://www.multimania.com/d2set/


                                                Prague, 29th May 1997

                                                Xavier Outhier

*/
// **************************************************************************
// **************************************************************************
//f

/*************************************************************************/
//ce
//ce  NAME OF FILE            : read_param.c++
//ce  VERSION                 : 0.4
//ce  DATE OF CREATION        : 07/07/1997 (Xavier)
//ce  LAST MODIFICATION ON    : 30/07/1997 (Xavier)
//ce  LAST CORRECTION ON      : 
//ce  COMPATIBILITY           : 1.0
//ce  STAGE OF VERIFICATION   : (see below)
//ce  DESCRIPTION OF THE FILE : Contains functions to read parameters that
//ce    files are in the directory ../DATA_IN
//ce
/*************************************************************************/

/*************************************************************************/
//cm
//cm  List  of modifications :
//cm
//cm   v0.1, 08/07/1997, Xavier:
//cm    # Add of read_position_fibre()
//cm             read_exit_pupil()
//cm             read_caract_light()
//cm
//cm   v0.2, 25/07/1997, Xavier:
//cm    # h_p_s replace with h_p_sx and h_p_sy
//cm
//cm   v0.3, 29/07/1997, Xavier:
//cm    # Suppression ofthe reading of angular misalignment. They are computed
//cm      from fr!
//cm
//cm   v0.4, 30/07/1997, Xavier:
//cm    # Remplacement of read_misalgn_fibre() by read_source_image()
//cm
/*************************************************************************/

/*************************************************************************/
//cc
//cc  List  of corrections :
//cc
/*************************************************************************/

/// "includes"
#include <stdlib.h>
#include <stdio.h>


// For C declaration funcions
#ifdef __cplusplus
extern "C" {
#endif

#ifdef __cplusplus
}
#endif

// For C++ declaration functions
#include <iostream.h>
#include <fstream.h>
#include "typedef.h"
#include "read_param.h"

//f

/// Extern vairables

// Intrinsic parameters of the fibre

  extern double n_core;
  extern double n_clad;
  extern double d_core;
  extern double d_clad;
  extern int fiber_type;

// Fibre position
  extern point2D eta_i;
  extern double z_i;

// Image source point position

  extern point2D eta_p;
  extern double z_p;

// Fibre orientation
  extern point3D fr;

// Caracteristics of window and of the source beam in the exit-pupil plane 
  extern struct bound_1D bound_x;
  extern struct bound_1D bound_y;
  extern double h_p_sx, h_p_sy;
  extern point2D X_OS_p;

// Caracteristics of light and surounding medium
  extern double lambda_0;
  extern double n_medium;

//f

/// "read_intrinsic_fibre"
/*************************************************************************/
//cf
//cf  Name of the function        : read_intrinsic_fibre
//cf  Version                     : 1.0
//cf  Date of creation            : 07/07/1997 (Xavier)
//cf  Last modification on        : 
//cf  Last correction on          : 
//cf  Compatibility               : 1.0
//cf  Verification Stage          : (see below)
//cf  Description of the function : read intrinsic paramters of the fiber:
//cf   the refractive index of the core, of the clad, diameters of the core 
//cf   and of the clad, type of fiber (step index or parabolic index).
//cf
/*************************************************************************/
void read_intrinsic_fibre(void)
{

// Opening of the input file

   ifstream intr_fib("../DATA_IN/intr_fibr.prm");
   if (!intr_fib) cout<<"unable to open '../DATA_IN/intr_fibr.prm' for input";

   intr_fib>>n_core
           >>n_clad
           >>d_core
           >>d_clad  
           >>fiber_type;

}
//f

/// "read_position_fibre"
/*************************************************************************/
//cf
//cf  Name of the function        : read_position_fibre
//cf  Version                     : 1.0
//cf  Date of creation            : 08/07/1997 (Xavier)
//cf  Last modification on        : 11/07/1997 (Xavier)
//cf  Last correction on          : 
//cf  Compatibility               : 1.0
//cf  Verification Stage          : (see below)
//cf    compilation with cc -c -fullwarn -Xcpluscomm ---------------> OK
//cf    linkage avec -lm -------------------------------------------> OK
//cf    execution et passage parametres ----------------------------> OK
//cf  Description of the function : read position paramters of the fiber:
//cf   the refractive index of the core, of the clad, diameters of the core 
//cf   and of the clad, type of fiber (step index or parabolic index).
//cf
/*************************************************************************/
void read_position_fibre(void)
{

// Opening of the input file

   ifstream pos_fib("../DATA_IN/pos_fibr.prm");
   if (!pos_fib) cout<<"unable to open '../DATA_IN/pos_fibr.prm' for input";


   pos_fib>>eta_i[0]
          >>eta_i[1]
          >>z_i
          >>fr[0]
          >>fr[1]
          >>fr[2];

}
//f

/// "read_source_image"
/*************************************************************************/
//cf
//cf  Name of the function        : read_source_image
//cf  Version                     : 1.0
//cf  Date of creation            : 30/07/1997 (Xavier)
//cf  Last modification on        : 
//cf  Last correction on          : 
//cf  Compatibility               : 1.0
//cf  Verification Stage          : (see below)
//cf  Description of the function : read the position of the source image
//cf   point (eta_p[0],eta_p[1],z_p).
//cf
/*************************************************************************/
void read_source_image(void)
{

// Opening of the input file

   ifstream misalign("../DATA_IN/source_image.prm");
   if (!misalign) cout<<"unable to open 'source_image.prm' for input";

   misalign>> eta_p[0]
           >> eta_p[1]
           >> z_p;

}
//f

/// "read_exit_pupil"
/*************************************************************************/
//cf
//cf  Name of the function        : read_exit_pupil
//cf  Version                     : 1.1
//cf  Date of creation            : 08/07/1997 (Xavier)
//cf  Last modification on        : 25/07/1997 (Xavier) 
//cf  Last correction on          : 
//cf  Compatibility               : 1.0
//cf  Verification Stage          : (see below)
//cf    compilation with cc -c -fullwarn -Xcpluscomm ---------------> OK
//cf    linkage avec -lm -------------------------------------------> OK
//cf    execution et passage parametres ----------------------------> OK
//cf  Description of the function : read position paramters of the fiber:
//cf   the refractive index of the core, of the clad, diameters of the core 
//cf   and of the clad, type of fiber (step index or parabolic index).
//cf
/*************************************************************************/
void read_exit_pupil(void)
{

// Opening of the input file

   ifstream exit_pup("../DATA_IN/exit_pupil.prm");
   if (!exit_pup) cout<<"unable to open '../DATA_IN/exit_pupil.prm' for input";


   exit_pup>> bound_x.num
           >> bound_y.num
           >> bound_x.min
           >> bound_x.max
           >> bound_y.min
           >> bound_y.max
           >> h_p_sx
           >> h_p_sy
           >> X_OS_p[0]
           >> X_OS_p[1];
}
//f

/// "read_caract_light"
/*************************************************************************/
//cf
//cf  Name of the function        : read_caract_light
//cf  Version                     : 1.0
//cf  Date of creation            : 08/07/1997 (Xavier)
//cf  Last modification on        : 
//cf  Last correction on          : 
//cf  Compatibility               : 1.0
//cf  Verification Stage          : (see below)
//cf    compilation with cc -c -fullwarn -Xcpluscomm ---------------> OK
//cf    linkage avec -lm -------------------------------------------> OK
//cf    execution et passage parametres ----------------------------> OK
//cf  Description of the function : read position paramters of the fiber:
//cf   the refractive index of the core, of the clad, diameters of the core 
//cf   and of the clad, type of fiber (step index or parabolic index).
//cf
/*************************************************************************/
void read_caract_light(void)
{

// Opening of the input file

   ifstream car_light("../DATA_IN/light.prm");
   if (!car_light) cout<<"unable to open '../DATA_IN/light.prm' for input";

   car_light>>lambda_0
            >>n_medium;

}
//f

/*************************************************************************/
//cl
//cl  End of file : read_param.c++
//cl
/*************************************************************************/
//f
